import matplotlib.pyplot as plt
import numpy as np
from matplotlib import rcParams
rcParams.update({'figure.autolayout': True})

#Script takes in the Energy, Charge Voltage, Nominal Voltage and Battery Efficiency. Produces 7 day long power schedules to be used in the "Power Simulation" on an Arbin battery cycler.

#Initialize battery property values
energy = 1#chg energy
chg_voltage = 1 #In (V)
# batt_cap = 1 #energy/chg_voltage #In (A*s = C)
batt_cap = energy/chg_voltage
nominal_voltage = 1
efficiency = 1 #from Ragone plots
batt_type = 'battery_chemistry'
batt_serial = 'serial_number'

#Initialize lists to be written to from template files
time = []
power = []
capacity= []
scaled_power = []
sec_in_a_day = 86400
cap_val = 0 

#Open each template file and assemble time, power, and capacity lists
#Reset capacity value every day
for day in range(1,8):
	st = 'template_day%r.txt' %(day)
	open_day_file = open(st)
	for line in open_day_file:
		time_val, power_val = line.split()
		power_val = float(power_val) * -1
		time.append(((day-1) * sec_in_a_day) + int(time_val))
		power.append(power_val)
		cap_val += ((float(power_val))/nominal_voltage)*(4)
		capacity.append(cap_val)
	cap_val = 0
open_day_file.close()

#Find the initial capacity percentage using the maximum and minimum capacity values
def init_perc(macks,minn,cap_perc):
	#returns necessary initial capacity percentage
	cap_perc = ((60)*(cap_perc - minn)/(macks - minn)) + 20
	return cap_perc

#Initial capacity percentage and max/min capacities (in terms of [A*s]=[C])
cap_max = max(capacity)
cap_min = min(capacity)
initial_cap_percentage = init_perc(cap_max,cap_min,0)
print 'The initial capacity percentage required is %r percent.' %float(('%.2f'%(initial_cap_percentage)))

#Convert max/min capacities to [A*h]
cap_max /= 3600
cap_min	/= 3600

#Power scaling factor, create a scaled power list
p_scale = ((80 - initial_cap_percentage)/(cap_max) * batt_cap)/100
for i in power:
	if i > 0:
		i *= p_scale
	else:
		i *= p_scale*efficiency	
	scaled_power.append(i)	

tm_scl_pwr = np.column_stack((time,scaled_power))
c_val = (initial_cap_percentage * batt_cap)/100 #percent
def init_lists():
	t = []
	temp = []
	cap = []
	return t, temp, cap

def iterine(i,t,temp,cap,days):
	global scaled_power
	global sec_in_a_day
	global nominal_voltage
	tm_scl_pwr[i][0] -= (days - 1)*(sec_in_a_day)
	t.append(tm_scl_pwr[i][0])
	temp.append(tm_scl_pwr[i])
	# c_val += ((scaled_power[i-1])/(3600 * voltage) * 4)
	# cap.append(c_val)
	return temp	

def write_to_file_full(days,i): #day 2-6
	t, temph, cap = init_lists()
	ts = '%s_%s_day%r.txt' %(batt_type,batt_serial,days)
	final_power_file = open(ts,'w+')
	if i % 21600 == 0:
		temph = iterine(i,t,temph,cap,days)
		i += 1		
	while i % 21600 != 0:
		temph = iterine(i,t,temph,cap,days)
		i += 1
	np.savetxt(final_power_file,temph,fmt='%.8f')
	final_power_file.close()
	return i


days = 1
i = 0
while days < 8:
		i = write_to_file_full(days,i)
		days += 1

tot_cap = 0
for i in scaled_power:
	tot_cap += abs(i)/voltage * 4
tot_cap /= 3600
print 'Total capacity in Ampere Hours = {}'.format(tot_cap)
bat_in_hour = (tot_cap/batt_cap)/168
print 'Number of batteries charge in an hour = {}'.format(bat_in_hour)
ep_ratio = 1/bat_in_hour
print 'Pseudo EP ratio should be {}'.format(ep_ratio)




